@@ -108,7 +108,7 @@ class TestDunder(unittest.TestCase):
         name = "xxxyyy"
         registration_name = "%s.%s" % (name, type_)
         info = ServiceInfo(
-            type_, registration_name, socket.inet_aton("10.0.1.2"), 80, 0, 0, b'', "ash-2.local."
+            type_, registration_name, 80, 0, 0, b'', "ash-2.local.", addresses=[socket.inet_aton("10.0.1.2")],
         )
 
         assert not info != info
@@ -121,7 +121,7 @@ class TestDunder(unittest.TestCase):
         info = ServiceInfo(
             type_=type_,
             name=registration_name,
-            address=socket.inet_aton("10.0.1.2"),
+            addresses=[socket.inet_aton("10.0.1.2")],
             port=80,
             server="ash-2.local.",
         )
@@ -318,6 +318,13 @@ class Names(unittest.TestCase):
         generated.add_question(question)
         r.DNSIncoming(generated.packet())
 
+    def test_extra_exceedingly_long_name(self):
+        generated = r.DNSOutgoing(r._FLAGS_QR_RESPONSE)
+        name = "%slocal." % ("part." * 4000)
+        question = r.DNSQuestion(name, r._TYPE_SRV, r._CLASS_IN)
+        generated.add_question(question)
+        r.DNSIncoming(generated.packet())
+
     def test_exceedingly_long_name_part(self):
         name = "%s.local." % ("a" * 1000)
         generated = r.DNSOutgoing(r._FLAGS_QR_RESPONSE)
@@ -355,12 +362,12 @@ class Names(unittest.TestCase):
 
         def send(out, addr=r._MDNS_ADDR, port=r._MDNS_PORT):
             """Sends an outgoing packet."""
-            packet = out.packet()
-            nonlocal longest_packet_len, longest_packet
-            if longest_packet_len < len(packet):
-                longest_packet_len = len(packet)
-                longest_packet = out
-            old_send(out, addr=addr, port=port)
+            for packet in out.packets():
+                nonlocal longest_packet_len, longest_packet
+                if longest_packet_len < len(packet):
+                    longest_packet_len = len(packet)
+                    longest_packet = out
+                old_send(out, addr=addr, port=port)
 
         # monkey patch the zeroconf send
         setattr(zc, "send", send)
@@ -374,6 +381,9 @@ class Names(unittest.TestCase):
 
         # wait until the browse request packet has maxed out in size
         sleep_count = 0
+        # we will never get to this large of a packet given the application-layer
+        # splitting of packets, but we still want to track the longest_packet_len
+        # for the debug message below
         while sleep_count < 100 and longest_packet_len < r._MAX_MSG_ABSOLUTE - 100:
             sleep_count += 1
             time.sleep(0.1)
@@ -386,8 +396,8 @@ class Names(unittest.TestCase):
         zeroconf.log.debug('sleep_count %d, sized %d', sleep_count, longest_packet_len)
 
         # now the browser has sent at least one request, verify the size
-        assert longest_packet_len <= r._MAX_MSG_ABSOLUTE
-        assert longest_packet_len >= r._MAX_MSG_ABSOLUTE - 100
+        assert longest_packet_len <= r._MAX_MSG_TYPICAL
+        assert longest_packet_len >= r._MAX_MSG_TYPICAL - 100
 
         # mock zeroconf's logger warning() and debug()
         from unittest.mock import patch
@@ -407,13 +417,11 @@ class Names(unittest.TestCase):
         call_counts = mocked_log_warn.call_count, mocked_log_debug.call_count
         # try to send an oversized packet
         zc.send(out)
-        assert mocked_log_warn.call_count == call_counts[0] + 1
-        assert mocked_log_debug.call_count == call_counts[0]
+        assert mocked_log_warn.call_count == call_counts[0]
         zc.send(out)
-        assert mocked_log_warn.call_count == call_counts[0] + 1
-        assert mocked_log_debug.call_count == call_counts[0] + 1
+        assert mocked_log_warn.call_count == call_counts[0]
 
-        # force a receive of an oversized packet
+        # force a receive of a packet
         packet = out.packet()
         s = zc._respond_sockets[0]
 
@@ -456,7 +464,14 @@ class Names(unittest.TestCase):
     def verify_name_change(self, zc, type_, name, number_hosts):
         desc = {'path': '/~paulsm/'}
         info_service = ServiceInfo(
-            type_, '%s.%s' % (name, type_), socket.inet_aton("10.0.1.2"), 80, 0, 0, desc, "ash-2.local."
+            type_,
+            '%s.%s' % (name, type_),
+            80,
+            0,
+            0,
+            desc,
+            "ash-2.local.",
+            addresses=[socket.inet_aton("10.0.1.2")],
         )
 
         # verify name conflict
@@ -736,7 +751,14 @@ class TestRegistrar(unittest.TestCase):
 
         desc = {'path': '/~paulsm/'}
         info = ServiceInfo(
-            type_, registration_name, socket.inet_aton("10.0.1.2"), 80, 0, 0, desc, "ash-2.local."
+            type_,
+            registration_name,
+            80,
+            0,
+            0,
+            desc,
+            "ash-2.local.",
+            addresses=[socket.inet_aton("10.0.1.2")],
         )
 
         # we are going to monkey patch the zeroconf send to check packet sizes
@@ -849,7 +871,14 @@ class ServiceTypesQuery(unittest.TestCase):
         zeroconf_registrar = Zeroconf(interfaces=['127.0.0.1'])
         desc = {'path': '/~paulsm/'}
         info = ServiceInfo(
-            type_, registration_name, socket.inet_aton("10.0.1.2"), 80, 0, 0, desc, "ash-2.local."
+            type_,
+            registration_name,
+            80,
+            0,
+            0,
+            desc,
+            "ash-2.local.",
+            addresses=[socket.inet_aton("10.0.1.2")],
         )
         zeroconf_registrar.register_service(info)
 
@@ -874,7 +903,14 @@ class ServiceTypesQuery(unittest.TestCase):
         zeroconf_registrar = Zeroconf(interfaces=['127.0.0.1'])
         desc = {'path': '/~paulsm/'}
         info = ServiceInfo(
-            type_, registration_name, socket.inet_pton(socket.AF_INET6, addr), 80, 0, 0, desc, "ash-2.local."
+            type_,
+            registration_name,
+            80,
+            0,
+            0,
+            desc,
+            "ash-2.local.",
+            addresses=[socket.inet_pton(socket.AF_INET6, addr)],
         )
         zeroconf_registrar.register_service(info)
 
@@ -898,7 +934,14 @@ class ServiceTypesQuery(unittest.TestCase):
         zeroconf_registrar = Zeroconf(ip_version=r.IPVersion.V6Only)
         desc = {'path': '/~paulsm/'}
         info = ServiceInfo(
-            type_, registration_name, socket.inet_aton("10.0.1.2"), 80, 0, 0, desc, "ash-2.local."
+            type_,
+            registration_name,
+            80,
+            0,
+            0,
+            desc,
+            "ash-2.local.",
+            addresses=[socket.inet_aton("10.0.1.2")],
         )
         zeroconf_registrar.register_service(info)
 
@@ -922,7 +965,14 @@ class ServiceTypesQuery(unittest.TestCase):
         zeroconf_registrar = Zeroconf(interfaces=['127.0.0.1'])
         desc = {'path': '/~paulsm/'}
         info = ServiceInfo(
-            discovery_type, registration_name, socket.inet_aton("10.0.1.2"), 80, 0, 0, desc, "ash-2.local."
+            discovery_type,
+            registration_name,
+            80,
+            0,
+            0,
+            desc,
+            "ash-2.local.",
+            addresses=[socket.inet_aton("10.0.1.2")],
         )
         zeroconf_registrar.register_service(info)
 
@@ -1028,7 +1078,14 @@ class ListenerTest(unittest.TestCase):
             properties['prop_blank'] = b'an updated string'
             desc.update(properties)
             info_service = ServiceInfo(
-                subtype, registration_name, socket.inet_aton("10.0.1.2"), 80, 0, 0, desc, "ash-2.local."
+                subtype,
+                registration_name,
+                80,
+                0,
+                0,
+                desc,
+                "ash-2.local.",
+                addresses=[socket.inet_aton("10.0.1.2")],
             )
             zeroconf_registrar.update_service(info_service)
             service_updated.wait(1)
@@ -1185,6 +1242,79 @@ class TestServiceBrowser(unittest.TestCase):
             zeroconf.close()
 
 
+class TestServiceBrowserMultipleTypes(unittest.TestCase):
+    def test_update_record(self):
+
+        service_names = ['name._type._tcp.local.', 'name._type._udp.local']
+        service_types = ['_type._tcp.local.', '_type._udp.local.']
+
+        service_added_count = 0
+        service_removed_count = 0
+        service_add_event = Event()
+        service_removed_event = Event()
+
+        class MyServiceListener(r.ServiceListener):
+            def add_service(self, zc, type_, name) -> None:
+                nonlocal service_added_count
+                service_added_count += 1
+                if service_added_count == 2:
+                    service_add_event.set()
+
+            def remove_service(self, zc, type_, name) -> None:
+                nonlocal service_removed_count
+                service_removed_count += 1
+                if service_removed_count == 2:
+                    service_removed_event.set()
+
+        def mock_incoming_msg(
+            service_state_change: r.ServiceStateChange, service_type: str, service_name: str
+        ) -> r.DNSIncoming:
+            generated = r.DNSOutgoing(r._FLAGS_QR_RESPONSE)
+
+            if service_state_change == r.ServiceStateChange.Removed:
+                ttl = 0
+            else:
+                ttl = 120
+
+            generated.add_answer_at_time(
+                r.DNSPointer(service_type, r._TYPE_PTR, r._CLASS_IN, ttl, service_name), 0
+            )
+            return r.DNSIncoming(generated.packet())
+
+        zeroconf = r.Zeroconf(interfaces=['127.0.0.1'])
+        service_browser = r.ServiceBrowser(zeroconf, service_types, listener=MyServiceListener())
+
+        try:
+            wait_time = 3
+
+            # both services added
+            zeroconf.handle_response(
+                mock_incoming_msg(r.ServiceStateChange.Added, service_types[0], service_names[0])
+            )
+            zeroconf.handle_response(
+                mock_incoming_msg(r.ServiceStateChange.Added, service_types[1], service_names[1])
+            )
+            service_add_event.wait(wait_time)
+            assert service_added_count == 2
+            assert service_removed_count == 0
+
+            # both services removed
+            zeroconf.handle_response(
+                mock_incoming_msg(r.ServiceStateChange.Removed, service_types[0], service_names[0])
+            )
+            zeroconf.handle_response(
+                mock_incoming_msg(r.ServiceStateChange.Removed, service_types[1], service_names[1])
+            )
+            service_removed_event.wait(wait_time)
+            assert service_added_count == 2
+            assert service_removed_count == 2
+
+        finally:
+            service_browser.cancel()
+            zeroconf.remove_all_service_listeners()
+            zeroconf.close()
+
+
 def test_backoff():
     got_query = Event()
 
@@ -1312,7 +1442,9 @@ def test_integration():
 
     zeroconf_registrar = Zeroconf(interfaces=['127.0.0.1'])
     desc = {'path': '/~paulsm/'}
-    info = ServiceInfo(type_, registration_name, socket.inet_aton("10.0.1.2"), 80, 0, 0, desc, "ash-2.local.")
+    info = ServiceInfo(
+        type_, registration_name, 80, 0, 0, desc, "ash-2.local.", addresses=[socket.inet_aton("10.0.1.2")]
+    )
     zeroconf_registrar.register_service(info)
 
     try:
@@ -1351,45 +1483,17 @@ def test_multiple_addresses():
     address_parsed = "10.0.1.2"
     address = socket.inet_aton(address_parsed)
 
-    # Old way
-    info = ServiceInfo(type_, registration_name, address, 80, 0, 0, desc, "ash-2.local.")
-
-    assert info.address == address
-    assert info.addresses == [address]
-
-    # Updating works
-    address2 = socket.inet_aton("10.0.1.3")
-    info.address = address2
-
-    assert info.address == address2
-    assert info.addresses == [address2]
-
-    info.address = None
-
-    assert info.address is None
-    assert info.addresses == []
-
-    info.addresses = [address2]
-
-    assert info.address == address2
-    assert info.addresses == [address2]
-
-    # Compatibility way
-    info = ServiceInfo(type_, registration_name, [address, address], 80, 0, 0, desc, "ash-2.local.")
-
-    assert info.addresses == [address, address]
-
     # New kwarg way
-    info = ServiceInfo(
-        type_, registration_name, None, 80, 0, 0, desc, "ash-2.local.", addresses=[address, address]
-    )
+    info = ServiceInfo(type_, registration_name, 80, 0, 0, desc, "ash-2.local.", addresses=[address, address])
 
     assert info.addresses == [address, address]
 
     if socket.has_ipv6 and not os.environ.get('SKIP_IPV6'):
         address_v6_parsed = "2001:db8::1"
         address_v6 = socket.inet_pton(socket.AF_INET6, address_v6_parsed)
-        info = ServiceInfo(type_, registration_name, [address, address_v6], 80, 0, 0, desc, "ash-2.local.")
+        info = ServiceInfo(
+            type_, registration_name, 80, 0, 0, desc, "ash-2.local.", addresses=[address, address_v6],
+        )
         assert info.addresses == [address]
         assert info.addresses_by_version(r.IPVersion.All) == [address, address_v6]
         assert info.addresses_by_version(r.IPVersion.V4Only) == [address]
@@ -1410,7 +1514,9 @@ def test_ptr_optimization():
     registration_name = "%s.%s" % (name, type_)
 
     desc = {'path': '/~paulsm/'}
-    info = ServiceInfo(type_, registration_name, socket.inet_aton("10.0.1.2"), 80, 0, 0, desc, "ash-2.local.")
+    info = ServiceInfo(
+        type_, registration_name, 80, 0, 0, desc, "ash-2.local.", addresses=[socket.inet_aton("10.0.1.2")]
+    )
 
     # we are going to monkey patch the zeroconf send to check packet sizes
     old_send = zc.send
